#include "mazeclass.as"

// Hexagonal Maze - Jim Bumgardner
//


// #include "bee_routines.as"

PW = 500;   // pixel width
PLM = 8;		// pixel left margin
PTM = PLM*2;		// pixel top margin
kNbrRows = 15; // 11;		// also try 19x22	 - nbrRows must be ODD (for vertical symmetry, and
												// cause the maze assumes the center row is the longest)
kMaxCols = 13; // 13;


WU = (PW-PLM*2) / (kMaxCols);	// width of hexagon

function HexMaze(nbrRows,maxCols)
{
	this.startRow = new Array(nbrRows);
	this.rowLen = new Array(nbrRows);
	var n = 0;
	this.centerRow = Math.floor(nbrRows/2);
	// trace("Center Row " + this.centerRow);
	var curRowLen = maxCols-this.centerRow;
	for (var i = 0; i < nbrRows; ++i) 
	{
		this.rowLen[i] = curRowLen;
		this.startRow[i] = n;
		n += curRowLen;
		if (i < this.centerRow)
			curRowLen++;
		else
			curRowLen--;
	}
	// trace("Max Cells = " + n);
	// trace("WU = " + WU);
	this.maxCells = n;
	this.nbrRows = nbrRows;
	this.maxCols = maxCols;
	this.digMethod = 2;	// use 1 for "snakier"

	this.idleReport = spinDice;
	this.completionFunc = FinishMaze;

}

HexMaze.prototype = new Maze();

HexMaze.prototype.pokeEntryHoles = function()
{
	this.cells[0].clearEdge(0);
	this.cells[0].clearEdge(1);
	this.cells[0].clearEdge(2);
	this.cells[this.maxCells-1].clearEdge(3);
	this.cells[this.maxCells-1].clearEdge(4);
	this.cells[this.maxCells-1].clearEdge(5);
}


HexMaze.prototype.getNbrEdges = function(addr)
{
	return 6;
}

HexMaze.prototype.getMaxCells = function()
{
	return this.maxCells;
}

HexMaze.prototype.getRowFromAddr = function(cellAddr)
{
   for (var i = 1; i < this.nbrRows; ++i)
    {
      if (cellAddr < this.startRow[i])
        return i-1;
    }
	  return this.nbrRows-1;
}

HexMaze.prototype.findNeighbors = function(cellAddr)
{
	var vec = new CellList(this.getNbrEdges(cellAddr));

	var row = this.getRowFromAddr(cellAddr);
	var col = cellAddr - this.startRow[row];

	// Edge 0
	if (row > 0 && (row > this.centerRow || col < this.rowLen[row]-1))
	{
		vec.cells[0] = this.cells[this.startRow[row-1] + col + (row > this.centerRow? 1 : 0)];
	}

	// Edge 1
	if (row > 0 && (row > this.centerRow || col > 0))
	{
		vec.cells[1] = this.cells[this.startRow[row-1] + col + (row > this.centerRow? 0 : -1)];
	}

	// Edge 2
	if (col > 0)
	{
		vec.cells[2] = this.cells[cellAddr-1];
	}
	
	// Edge 3
	if (row < this.nbrRows-1 && (row < this.centerRow || col > 0))
	{
		vec.cells[3] = this.cells[this.startRow[row+1] + col + (row < this.centerRow? 0 : -1)];
	}

	// Edge 4
	if (row < this.nbrRows-1 && (row < this.centerRow || col < this.rowLen[row]-1))
	{
		vec.cells[4] = this.cells[this.startRow[row+1] + col + (row < this.centerRow? 1 : 0)];
	}

	// Edge 5
	if (col < this.rowLen[row]-1) 
	{
		vec.cells[5] = this.cells[cellAddr+1];
	}
	return vec;
}

MovieClip.prototype.drawLine = function(x1,y1,x2,y2)
{
		this.moveTo(x1, y1);
		this.lineTo(x2, y2);
}

HEXR = WU/Math.sqrt(3);	// Hegagon Radius

TYS = HEXR*1.5;					// tile y spacing

TXR = WU/2;							// tile x radius
TYR = HEXR;							// tile y radius

LM = PLM+WU/2;
TM = PTM+HEXR/2;

arcX = new Array(7);
arcY = new Array(7);

for (var i = 0; i < 7; ++i) 
{
	arcX[i] = Math.cos((30+i*60)*Math.PI/180);
	arcY[i] = -Math.sin((30+i*60)*Math.PI/180);
}

MovieClip.prototype.drawGradHex = function(ox,oy)
{
  var colors = [0xFFFFFF, 0xC3B710];
	var g_ratios = [0x00,0xFF];
	var g_alphas = [100,100];
	var g_matrix = { matrixType:"box", x:ox-HEXR, y:oy-HEXR, w:HEXR*2, h:HEXR*2, r: 0 }

	this.moveTo(ox+arcX[0]*HEXR,		oy+arcY[0]*HEXR);

 	mc.beginGradientFill("radial",colors,g_alphas,g_ratios,g_matrix);

  for (var i = 1; i < 6; ++i)
  {
		  mc.lineTo(ox+arcX[i]*HEXR,		oy+arcY[i]*HEXR,
					 			  ox+arcX[i+1]*HEXR,	oy+arcY[i+1]*HEXR);
  }

  mc.endFill()
}

HexMaze.prototype.getX = function(addr)
{
	var row = this.getRowFromAddr(addr);
	var col = addr - this.startRow[row];
	if (row <= this.centerRow)
		return LM+col*WU+(this.centerRow-row)*WU/2;
	else
		return LM+col*WU+(row-this.centerRow)*WU/2;
}

HexMaze.prototype.getY = function(addr)
{
	var row = this.getRowFromAddr(addr);
	var col = addr - this.startRow[row];
	return TM+row*TYS;
}

HexMaze.prototype.drawCell = function(mc, cell)
{
	var row = this.getRowFromAddr(cell.address);
	var col = cell.address - this.startRow[row];
	var oy = TM+row*TYS;
	var ox;
	if (row <= this.centerRow)
		ox = LM+col*WU+(this.centerRow-row)*WU/2;
	else
		ox = LM+col*WU+(row-this.centerRow)*WU/2;

	// trace("cell addr " + cell.address + " r,c " + row + "," + col + " ox,oy " + ox + "," + oy);

	mc.lineStyle(0,0,0);
	mc.drawGradHex(ox,oy);

	mc.lineStyle(2, 0x000000, 100);

  for (var i = 0; i < 6; ++i)
  {
		if ((cell.edgeFlags & (1 << i)) > 0)
		{
		  mc.drawLine(ox+arcX[i]*HEXR,		oy+arcY[i]*HEXR,
					 			  ox+arcX[i+1]*HEXR,	oy+arcY[i+1]*HEXR);
		}
  }
}

function drawMyMaze()
{
	// trace("Draw My Maze " + this.maze.cells.length + " LM = " + LM);
	mc.clear();
	mc.lineStyle(2, 0x000000, 100);
	for (var i = this.maze.cells.length; --i >= 0;)
	{
		this.maze.drawCell(this, this.maze.cells[i]);
	}
	this.onEnterFrame = undefined;
}

function spinDice(r)
{
	dice_mc._rotation = r*360;	
}

function animButtonGrow()
{
	delta = this.tScale - this._xscale;
	if (delta >= 1) 
	{
		this._xscale += delta/2;
		this._yscale += delta/2;
	}
	else {
		this._xscale = this.tScale;
		this._yscale = this.tScale;
		this.onEnterFrame = undefined;
	}
}

function growButton()
{
	this.tScale = 125;
	this.active = true;
	this.onEnterFrame = animButtonGrow;
}

function shrinkButton()
{
	this.tScale = 100;
	this.active = false;
	this.onEnterFrame = animButtonGrow;
}

function FinishMaze()
{
	mc.onEnterFrame = drawMyMaze;
	bee_mc.setup();
}

function resetMaze()
{
	if (this.active)
		mc.maze.init();
}



function init()
{
	this.mc = this.createEmptyMovieClip("maze_mc", 0);
	this.mc._x = 0;
	this.mc._y = 0;
	this.mc.maze = new HexMaze(kNbrRows, kMaxCols);
	this.mc.maze.init();

	this.dice_mc.onMouseUp = resetMaze;
	this.dice_mc.onRollOver = growButton;
	this.dice_mc.onRollOut =  shrinkButton;
}

init();

#include "bee_routines.as"

function nextCellNav(curAddr)	// Bee function
{
	// trace("next cell nav: " + curAddr);
	if (curAddr == this.goalCellAddress)
	{
		trace("Maze Complete!!");
		mc.maze.init();	// start a new one...
		return;
	}
	var vec = mc.maze.findOpenAccess(curAddr);
	var nextCell;
	var	cell = mc.maze.cells[curAddr];
	cell.visited = true;
	if (vec.containsNonEmpties()) 
	{
		// trace("onward from " + curAddr);
		mc.maze.cstack.push(curAddr);
		nextCell = vec.cells[vec.getRandomEdge()];
	}
	else {
		// trace("dead end!");
		nextCell = mc.maze.cells[mc.maze.cstack.pop()];
	}
	var ox = mc.maze.getX(nextCell.address);
	var oy = mc.maze.getY(nextCell.address);
	this.setTarget(ox, oy, nextCellNav, nextCell.address, BEE_WALK);
}


Bee.prototype.setup = function()
{
	trace("Setting up Bee");
	this.goalCellAddress = mc.maze.maxCells-1;
	mc.maze.clearVisited();
	ox = mc.maze.getX(0);
	oy = mc.maze.getY(0);
	this.setTarget(ox, oy, nextCellNav, 0, BEE_FLY);
}

function beeblink(bee)
{
	bee.head.gotoAndPlay(1);
	clearInterval(_root.blinkH);
	blinkH = setInterval(beeblink, random(10000), bee);
}


this.bee_mc._xscale = 20;
this.bee_mc._yscale = 20;
this.bee_mc._x = -100;
this.bee_mc._y = TM;
_root.blinkH = setInterval(beeblink, random(1000),this.bee_mc);
this.bee_mc.setState(BEE_IDLE);
